let serverUrl = "http://localhost:5000/submit";

function loadServerUrl() {
    return browser.storage.sync.get("serverUrl").then((result) => {
        if (result.serverUrl) {
            serverUrl = result.serverUrl;
            console.log("URL сервера загружен из настроек:", serverUrl);
        } else {
            return fetch(browser.runtime.getURL("config.json"))
                .then(response => {
                    if (!response.ok) {
                        throw new Error(`Ошибка загрузки config.json: ${response.statusText}`);
                    }
                    return response.json();
                })
                .then(config => {
                    serverUrl = config.FLASK_SERVER_URL;
                    console.log("URL сервера успешно загружен:", serverUrl);
                });
        }
    }).catch((error) => {
        console.error("Ошибка загрузки URL сервера:", error);
    });
}

loadServerUrl();


let clientId = null;

function generateUUID() {
    return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, (c) => {
        const r = (Math.random() * 16) | 0;
        const v = c === 'x' ? r : (r & 0x3) | 0x8;
        return v.toString(16);
    });
}

browser.storage.sync.get("clientId").then((result) => {
    if (result.clientId) {
        clientId = result.clientId;
        console.log("ID клиента загружен:", clientId);
    } else {
        clientId = generateUUID();
        browser.storage.sync.set({ clientId }).then(() => {
            console.log("ID клиента создан и сохранен:", clientId);
        });
    }
}).catch((error) => {
    console.error("Ошибка загрузки ID клиента:", error);
});


function createEventJSON(eventType, data) {
    const timeStamp = new Date().toISOString();
    return {
        "@version": 1,
        "type": `browserext.${eventType}`,
        "TimeStamp": timeStamp,
        "data": {...data, clientId },
        "@timestamp": timeStamp
    };
}


async function sendDataToServer(eventData) {
    if (!serverUrl) {
        console.error("URL сервера не задан. Проверьте настройки.");
        return;
    }
    try {
        const response = await fetch(serverUrl, {
            method: 'POST',
            headers: { 'Content-Type': 'application/json' },
            body: JSON.stringify(eventData),
        });

        const responseData = await response.json();
        if (response.ok) {
            console.log("Данные успешно отправлены на сервер:", responseData);
        } else {
            console.error("Ошибка при отправке данных на сервер:", response.status, responseData);
        }
    } catch (error) {
        console.error("Ошибка подключения к серверу:", error);
    }
}


document.addEventListener("keydown", (event) => {
    const key = event.key;
    const code = event.code;
    let description;

    if (key.match(/^\p{Number}$/u)) {
        description = "number";
    } else if (key.match(/^\p{Letter}$/u)) {
        description = "letter";
    } else if (key.match(/^\p{Punctuation}$|^\p{Symbol}$/u)) {
        description = "symbol";
    } else {
        description = `Клавиша: ${key} Код: ${code}`;
    }

    const eventData = createEventJSON('keydown', { description });
    console.log("Событие keydown:", JSON.stringify(eventData));
    sendDataToServer(eventData);
});


document.addEventListener("mousedown", (event) => {
    const eventData = createEventJSON('mousedown', { button: event.button, description: `Клик мышью: ${event.button}` });
    console.log("Событие mousedown:", JSON.stringify(eventData));
    sendDataToServer(eventData);
});


document.addEventListener("click", (event) => {
    const eventType = 'click';
    const clickedElement = event.target;
    const tag = clickedElement.tagName;
    const id = clickedElement.id || "отсутствует";
    const className = clickedElement.className || "отсутствует";

    let description = "Клик по элементу";
    let data = { tag, id, className };

    if (tag === 'A') {
        const href = clickedElement.href;
        data.href = href;
        description = "Клик по ссылке";

        localStorage.setItem('clickedElementInfo', JSON.stringify(data));
        navigateWithPromise(href);
    } else if (tag === 'INPUT') {
        const placeholder = clickedElement.placeholder || "отсутствует";
        data.placeholder = placeholder;
        description = "Клик по полю ввода";
    }

    data.description = description;

    const eventData = createEventJSON(eventType, data);
    console.log("Событие click:", JSON.stringify(eventData));
    sendDataToServer(eventData);
});


function navigateWithPromise(href) {
    return new Promise((resolve) => {
        window.location.href = href;
        window.addEventListener('load', resolve);
    });
}


document.addEventListener("copy", (event) => {
    const selectedText = window.getSelection().toString();
    const pageUrl = window.location.href;

    let outputText = selectedText.length > 50 ? selectedText.substring(0, 49) + '…' : selectedText;

    if (selectedText.length > 0) {
        const eventData = createEventJSON('copy', { selectedText: outputText, pageUrl, description: "Скопирован текст" });
        console.log("Событие copy:", JSON.stringify(eventData));
        sendDataToServer(eventData);
    }
});